Class: Resque::Worker
- Inherits:
-
Object
- Object
- Resque::Worker
- Extended by:
- Helpers
- Defined in:
- lib/resque/worker.rb
Overview
A Resque Worker processes jobs. On platforms that support fork(2), the worker will fork off a child to process each job. This ensures a clean slate when beginning the next job and cuts down on gradual memory growth as well as low level failures.
It also ensures workers are always listening to signals from you, their master, and can react accordingly.
Instance Attribute Summary collapse
-
#cant_fork ⇒ Object
Boolean indicating whether this worker can or can not fork.
-
#run_at_exit_hooks ⇒ Object
When set to true, forked workers will exit with
exit, calling anyat_exitcode handlers that have been registered in the application. -
#term_child ⇒ Object
decide whether to use new_kill_child logic.
-
#term_timeout ⇒ Object
Returns the value of attribute term_timeout.
-
#to_s ⇒ Object
(also: #id)
The string representation is the same as the id for this worker instance.
Class Method Summary collapse
-
.all ⇒ Object
Returns an array of all worker objects.
-
.attach(worker_id) ⇒ Object
Alias of
find. -
.exists?(worker_id) ⇒ Boolean
Given a string worker id, return a boolean indicating whether the worker exists.
-
.find(worker_id) ⇒ Object
Returns a single worker object.
-
.working ⇒ Object
Returns an array of all worker objects currently processing jobs.
Instance Method Summary collapse
-
#==(other) ⇒ Object
Is this worker the same as another worker?.
-
#done_working ⇒ Object
Called when we are done working - clears our
working_onstate and tells Redis we processed a job. -
#enable_gc_optimizations ⇒ Object
Enables GC Optimizations if you’re running REE.
-
#failed ⇒ Object
How many failed jobs has this worker seen? Returns an int.
-
#failed! ⇒ Object
Tells Redis we’ve failed a job.
-
#fork(job, &block) ⇒ Object
Not every platform supports fork.
-
#hostname ⇒ Object
chomp’d hostname of this machine.
-
#idle? ⇒ Boolean
Boolean - true if idle, false if not.
-
#initialize(*queues) ⇒ Worker
constructor
Workers should be initialized with an array of string queue names.
- #inspect ⇒ Object
-
#job ⇒ Object
(also: #processing)
Returns a hash explaining the Job we’re currently processing, if any.
-
#kill_child ⇒ Object
Kills the forked child immediately, without remorse.
-
#linux_worker_pids ⇒ Object
Find Resque worker pids on Linux and OS X.
-
#log(message) ⇒ Object
Log a message to Resque.logger can’t use alias_method since info/debug are private methods.
- #log!(message) ⇒ Object
- #logger_severity_deprecation_warning ⇒ Object
-
#new_kill_child ⇒ Object
Kills the forked child immediately with minimal remorse.
-
#pause_processing ⇒ Object
Stop processing jobs after the current one has completed (if we’re currently running one).
-
#paused? ⇒ Boolean
are we paused?.
-
#perform(job) ⇒ Object
Processes a given job in the child.
-
#pid ⇒ Object
Returns Integer PID of running worker.
-
#process(job = nil, &block) ⇒ Object
DEPRECATED.
-
#processed ⇒ Object
How many jobs has this worker processed? Returns an int.
-
#processed! ⇒ Object
Tell Redis we’ve processed a job.
-
#procline(string) ⇒ Object
Given a string, sets the procline ($0) and logs.
-
#prune_dead_workers ⇒ Object
Looks for any workers which should be running on this server and, if they’re not, removes them from Redis.
-
#queues ⇒ Object
Returns a list of queues to use when searching for a job.
-
#reconnect ⇒ Object
Reconnect to Redis to avoid sharing a connection with the parent, retry up to 3 times with increasing delay before giving up.
-
#register_signal_handlers ⇒ Object
Registers the various signal handlers a worker responds to.
-
#register_worker ⇒ Object
Registers ourself as a worker.
-
#reserve ⇒ Object
Attempts to grab a job off one of the provided queues.
-
#run_hook(name, *args) ⇒ Object
Runs a named hook, passing along any arguments.
-
#shutdown ⇒ Object
Schedule this worker for shutdown.
-
#shutdown! ⇒ Object
Kill the child and shutdown immediately.
-
#shutdown? ⇒ Boolean
Should this worker shutdown as soon as current job is finished?.
-
#solaris_worker_pids ⇒ Object
Find Resque worker pids on Solaris.
-
#started ⇒ Object
What time did this worker start? Returns an instance of
Time. -
#started! ⇒ Object
Tell Redis we’ve started.
-
#startup ⇒ Object
Runs all the methods needed when a worker begins its lifecycle.
-
#state ⇒ Object
Returns a symbol representing the current worker state, which can be either :working or :idle.
-
#unpause_processing ⇒ Object
Start processing jobs again after a pause.
- #unregister_signal_handlers ⇒ Object
-
#unregister_worker(exception = nil) ⇒ Object
Unregisters ourself as a worker.
-
#validate_queues ⇒ Object
A worker must be given a queue, otherwise it won’t know what to do with itself.
-
#verbose ⇒ Object
Deprecated legacy methods for controlling the logging threshhold Use Resque.logger.level now, e.g.:.
- #verbose=(value) ⇒ Object
- #very_verbose ⇒ Object
- #very_verbose=(value) ⇒ Object
- #will_fork? ⇒ Boolean
-
#windows_worker_pids ⇒ Object
Returns an Array of string pids of all the other workers on this machine.
-
#work(interval = 5.0, &block) ⇒ Object
This is the main workhorse method.
-
#worker_pids ⇒ Object
Returns an Array of string pids of all the other workers on this machine.
-
#working? ⇒ Boolean
Boolean - true if working, false if not.
-
#working_on(job) ⇒ Object
Given a job, tells Redis we’re working on it.
Methods included from Helpers
classify, constantize, decode, encode, redis
Methods included from Logging
debug, error, fatal, info, log, warn
Constructor Details
#initialize(*queues) ⇒ Worker
Workers should be initialized with an array of string queue names. The order is important: a Worker will check the first queue given for a job. If none is found, it will check the second queue name given. If a job is found, it will be processed. Upon completion, the Worker will again check the first queue given, and so forth. In this way the queue list passed to a Worker on startup defines the priorities of queues.
If passed a single “*”, this Worker will operate on all queues in alphabetical order. Queues can be dynamically added or removed without needing to restart workers using this method.
96 97 98 99 100 101 |
# File 'lib/resque/worker.rb', line 96 def initialize(*queues) @queues = queues.map { |queue| queue.to_s.strip } @shutdown = nil @paused = nil validate_queues end |
Instance Attribute Details
#cant_fork ⇒ Object
Boolean indicating whether this worker can or can not fork. Automatically set if a fork(2) fails.
18 19 20 |
# File 'lib/resque/worker.rb', line 18 def cant_fork @cant_fork end |
#run_at_exit_hooks ⇒ Object
When set to true, forked workers will exit with exit, calling any at_exit code handlers that have been registered in the application. Otherwise, forked workers exit with exit!
27 28 29 |
# File 'lib/resque/worker.rb', line 27 def run_at_exit_hooks @run_at_exit_hooks end |
#term_child ⇒ Object
decide whether to use new_kill_child logic
23 24 25 |
# File 'lib/resque/worker.rb', line 23 def term_child @term_child end |
#term_timeout ⇒ Object
Returns the value of attribute term_timeout.
20 21 22 |
# File 'lib/resque/worker.rb', line 20 def term_timeout @term_timeout end |
#to_s ⇒ Object Also known as: id
The string representation is the same as the id for this worker instance. Can be used with Worker.find.
561 562 563 |
# File 'lib/resque/worker.rb', line 561 def to_s @to_s ||= "#{hostname}:#{pid}:#{@queues.join(',')}" end |
Class Method Details
.all ⇒ Object
Returns an array of all worker objects.
32 33 34 |
# File 'lib/resque/worker.rb', line 32 def self.all Array(redis.smembers(:workers)).map { |id| find(id) }.compact end |
.attach(worker_id) ⇒ Object
Alias of find
75 76 77 |
# File 'lib/resque/worker.rb', line 75 def self.attach(worker_id) find(worker_id) end |
.exists?(worker_id) ⇒ Boolean
Given a string worker id, return a boolean indicating whether the worker exists
81 82 83 |
# File 'lib/resque/worker.rb', line 81 def self.exists?(worker_id) redis.sismember(:workers, worker_id) end |
.find(worker_id) ⇒ Object
Returns a single worker object. Accepts a string id.
63 64 65 66 67 68 69 70 71 72 |
# File 'lib/resque/worker.rb', line 63 def self.find(worker_id) if exists? worker_id queues = worker_id.split(':')[-1].split(',') worker = new(*queues) worker.to_s = worker_id worker else nil end end |
.working ⇒ Object
Returns an array of all worker objects currently processing jobs.
38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 |
# File 'lib/resque/worker.rb', line 38 def self.working names = all return [] unless names.any? names.map! { |name| "worker:#{name}" } reportedly_working = {} begin reportedly_working = redis.mapped_mget(*names).reject do |key, value| value.nil? || value.empty? end rescue Redis::Distributed::CannotDistribute names.each do |name| value = redis.get name reportedly_working[name] = value unless value.nil? || value.empty? end end reportedly_working.keys.map do |key| find key.sub("worker:", '') end.compact end |
Instance Method Details
#==(other) ⇒ Object
Is this worker the same as another worker?
551 552 553 |
# File 'lib/resque/worker.rb', line 551 def ==(other) to_s == other.to_s end |
#done_working ⇒ Object
Called when we are done working - clears our working_on state and tells Redis we processed a job.
487 488 489 490 |
# File 'lib/resque/worker.rb', line 487 def done_working processed! redis.del("worker:#{self}") end |
#enable_gc_optimizations ⇒ Object
Enables GC Optimizations if you’re running REE. www.rubyenterpriseedition.com/faq.html#adapt_apps_for_cow
292 293 294 295 296 |
# File 'lib/resque/worker.rb', line 292 def enable_gc_optimizations if GC.respond_to?(:copy_on_write_friendly=) GC.copy_on_write_friendly = true end end |
#failed ⇒ Object
How many failed jobs has this worker seen? Returns an int.
504 505 506 |
# File 'lib/resque/worker.rb', line 504 def failed Stat["failed:#{self}"] end |
#failed! ⇒ Object
Tells Redis we’ve failed a job.
509 510 511 512 |
# File 'lib/resque/worker.rb', line 509 def failed! Stat << "failed" Stat << "failed:#{self}" end |
#fork(job, &block) ⇒ Object
Not every platform supports fork. Here we do our magic to determine if yours does.
256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 |
# File 'lib/resque/worker.rb', line 256 def fork(job,&block) return if @cant_fork # Only run before_fork hooks if we're actually going to fork # (after checking @cant_fork) run_hook :before_fork, job begin # IronRuby doesn't support `Kernel.fork` yet if Kernel.respond_to?(:fork) Kernel.fork &block if will_fork? else raise NotImplementedError end rescue NotImplementedError @cant_fork = true nil end end |
#hostname ⇒ Object
chomp’d hostname of this machine
567 568 569 |
# File 'lib/resque/worker.rb', line 567 def hostname @hostname ||= `hostname`.chomp end |
#idle? ⇒ Boolean
Boolean - true if idle, false if not
536 537 538 |
# File 'lib/resque/worker.rb', line 536 def idle? state == :idle end |
#inspect ⇒ Object
555 556 557 |
# File 'lib/resque/worker.rb', line 555 def inspect "#<Worker #{to_s}>" end |
#job ⇒ Object Also known as: processing
Returns a hash explaining the Job we’re currently processing, if any.
525 526 527 |
# File 'lib/resque/worker.rb', line 525 def job decode(redis.get("worker:#{self}")) || {} end |
#kill_child ⇒ Object
Kills the forked child immediately, without remorse. The job it is processing will not be completed.
362 363 364 365 366 367 368 369 370 371 372 |
# File 'lib/resque/worker.rb', line 362 def kill_child if @child log! "Killing child at #{@child}" if `ps -o pid,state -p #{@child}` Process.kill("KILL", @child) rescue nil else log! "Child #{@child} not found, restarting." shutdown end end end |
#linux_worker_pids ⇒ Object
Find Resque worker pids on Linux and OS X.
595 596 597 598 599 |
# File 'lib/resque/worker.rb', line 595 def linux_worker_pids `ps -A -o pid,command | grep "[r]esque" | grep -v "resque-web"`.split("\n").map do |line| line.split(' ')[0] end end |
#log(message) ⇒ Object
Log a message to Resque.logger can’t use alias_method since info/debug are private methods
625 626 627 |
# File 'lib/resque/worker.rb', line 625 def log() info() end |
#log!(message) ⇒ Object
629 630 631 |
# File 'lib/resque/worker.rb', line 629 def log!() debug() end |
#logger_severity_deprecation_warning ⇒ Object
673 674 675 676 677 678 679 680 |
# File 'lib/resque/worker.rb', line 673 def logger_severity_deprecation_warning return if $TESTING return if $warned_logger_severity_deprecation Kernel.warn "*** DEPRECATION WARNING: Resque::Worker#verbose and #very_verbose are deprecated. Please set Resque.logger.level instead" Kernel.warn "Called from: #{caller[0..5].join("\n\t")}" $warned_logger_severity_deprecation = true nil end |
#new_kill_child ⇒ Object
Kills the forked child immediately with minimal remorse. The job it is processing will not be completed. Send the child a TERM signal, wait 5 seconds, and then a KILL signal if it has not quit
377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 |
# File 'lib/resque/worker.rb', line 377 def new_kill_child if @child unless Process.waitpid(@child, Process::WNOHANG) log! "Sending TERM signal to child #{@child}" Process.kill("TERM", @child) (term_timeout.to_f * 10).round.times do |i| sleep(0.1) return if Process.waitpid(@child, Process::WNOHANG) end log! "Sending KILL signal to child #{@child}" Process.kill("KILL", @child) else log! "Child #{@child} already quit." end end rescue SystemCallError log! "Child #{@child} already quit and reaped." end |
#pause_processing ⇒ Object
Stop processing jobs after the current one has completed (if we’re currently running one).
403 404 405 406 |
# File 'lib/resque/worker.rb', line 403 def pause_processing log "USR2 received; pausing job processing" @paused = true end |
#paused? ⇒ Boolean
are we paused?
397 398 399 |
# File 'lib/resque/worker.rb', line 397 def paused? @paused end |
#perform(job) ⇒ Object
Processes a given job in the child.
192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 |
# File 'lib/resque/worker.rb', line 192 def perform(job) begin run_hook :after_fork, job if will_fork? job.perform rescue Object => e log "#{job.inspect} failed: #{e.inspect}" begin job.fail(e) rescue Object => e log "Received exception when reporting failure: #{e.inspect}" end failed! else log "done: #{job.inspect}" ensure yield job if block_given? end end |
#pid ⇒ Object
Returns Integer PID of running worker
572 573 574 |
# File 'lib/resque/worker.rb', line 572 def pid @pid ||= Process.pid end |
#process(job = nil, &block) ⇒ Object
DEPRECATED. Processes a single job. If none is given, it will try to produce one. Usually run in the child.
181 182 183 184 185 186 187 188 189 |
# File 'lib/resque/worker.rb', line 181 def process(job = nil, &block) return unless job ||= reserve job.worker = self working_on job perform(job, &block) ensure done_working end |
#processed ⇒ Object
How many jobs has this worker processed? Returns an int.
493 494 495 |
# File 'lib/resque/worker.rb', line 493 def processed Stat["processed:#{self}"] end |
#processed! ⇒ Object
Tell Redis we’ve processed a job.
498 499 500 501 |
# File 'lib/resque/worker.rb', line 498 def processed! Stat << "processed" Stat << "processed:#{self}" end |
#procline(string) ⇒ Object
Given a string, sets the procline ($0) and logs. Procline is always in the format of:
resque-VERSION: STRING
618 619 620 621 |
# File 'lib/resque/worker.rb', line 618 def procline(string) $0 = "resque-#{Resque::Version}: #{string}" log! $0 end |
#prune_dead_workers ⇒ Object
Looks for any workers which should be running on this server and, if they’re not, removes them from Redis.
This is a form of garbage collection. If a server is killed by a hard shutdown, power failure, or something else beyond our control, the Resque workers will not die gracefully and therefore will leave stale state information in Redis.
By checking the current Redis state against the actual environment, we can determine if Redis is old and clean it up a bit.
424 425 426 427 428 429 430 431 432 433 434 |
# File 'lib/resque/worker.rb', line 424 def prune_dead_workers all_workers = Worker.all known_workers = worker_pids unless all_workers.empty? all_workers.each do |worker| host, pid, queues = worker.id.split(':') next unless host == hostname next if known_workers.include?(pid) log! "Pruning dead worker: #{worker}" worker.unregister_worker end end |
#queues ⇒ Object
Returns a list of queues to use when searching for a job. A splat (“*”) means you want every queue (in alpha order) - this can be useful for dynamically adding new queues.
250 251 252 |
# File 'lib/resque/worker.rb', line 250 def queues @queues.map {|queue| queue == "*" ? Resque.queues.sort : queue }.flatten.uniq end |
#reconnect ⇒ Object
Reconnect to Redis to avoid sharing a connection with the parent, retry up to 3 times with increasing delay before giving up.
231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 |
# File 'lib/resque/worker.rb', line 231 def reconnect tries = 0 begin redis.client.reconnect rescue Redis::BaseConnectionError if (tries += 1) <= 3 log "Error reconnecting to Redis; retrying" sleep(tries) retry else log "Error reconnecting to Redis; quitting" raise end end end |
#register_signal_handlers ⇒ Object
Registers the various signal handlers a worker responds to.
TERM: Shutdown immediately, stop processing jobs.
INT: Shutdown immediately, stop processing jobs.
QUIT: Shutdown after the current job has finished processing. USR1: Kill the forked child immediately, continue processing jobs. USR2: Don’t process any new jobs CONT: Start processing jobs again after a USR2
306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 |
# File 'lib/resque/worker.rb', line 306 def register_signal_handlers trap('TERM') { shutdown! } trap('INT') { shutdown! } begin trap('QUIT') { shutdown } if term_child trap('USR1') { new_kill_child } else trap('USR1') { kill_child } end trap('USR2') { pause_processing } trap('CONT') { unpause_processing } rescue ArgumentError warn "Signals QUIT, USR1, USR2, and/or CONT not supported." end log! "Registered signals" end |
#register_worker ⇒ Object
Registers ourself as a worker. Useful when entering the worker lifecycle on startup.
438 439 440 441 |
# File 'lib/resque/worker.rb', line 438 def register_worker redis.sadd(:workers, self) started! end |
#reserve ⇒ Object
Attempts to grab a job off one of the provided queues. Returns nil if no job can be found.
213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 |
# File 'lib/resque/worker.rb', line 213 def reserve queues.each do |queue| log! "Checking #{queue}" if job = Resque.reserve(queue) log! "Found job on #{queue}" return job end end nil rescue Exception => e log "Error reserving job: #{e.inspect}" log e.backtrace.join("\n") raise e end |
#run_hook(name, *args) ⇒ Object
Runs a named hook, passing along any arguments.
444 445 446 447 448 449 450 451 452 453 |
# File 'lib/resque/worker.rb', line 444 def run_hook(name, *args) return unless hooks = Resque.send(name) msg = "Running #{name} hooks" msg << " with #{args.inspect}" if args.any? log msg hooks.each do |hook| args.any? ? hook.call(*args) : hook.call end end |
#shutdown ⇒ Object
Schedule this worker for shutdown. Will finish processing the current job.
340 341 342 343 |
# File 'lib/resque/worker.rb', line 340 def shutdown log 'Exiting...' @shutdown = true end |
#shutdown! ⇒ Object
Kill the child and shutdown immediately.
346 347 348 349 350 351 352 353 |
# File 'lib/resque/worker.rb', line 346 def shutdown! shutdown if term_child new_kill_child else kill_child end end |
#shutdown? ⇒ Boolean
Should this worker shutdown as soon as current job is finished?
356 357 358 |
# File 'lib/resque/worker.rb', line 356 def shutdown? @shutdown end |
#solaris_worker_pids ⇒ Object
Find Resque worker pids on Solaris.
Returns an Array of string pids of all the other workers on this machine. Useful when pruning dead workers on startup.
605 606 607 608 609 610 611 612 613 |
# File 'lib/resque/worker.rb', line 605 def solaris_worker_pids `ps -A -o pid,comm | grep "[r]uby" | grep -v "resque-web"`.split("\n").map do |line| real_pid = line.split(' ')[0] pargs_command = `pargs -a #{real_pid} 2>/dev/null | grep [r]esque | grep -v "resque-web"` if pargs_command.split(':')[1] == " resque-#{Resque::Version}" real_pid end end.compact end |
#started ⇒ Object
What time did this worker start? Returns an instance of Time
515 516 517 |
# File 'lib/resque/worker.rb', line 515 def started redis.get "worker:#{self}:started" end |
#started! ⇒ Object
Tell Redis we’ve started
520 521 522 |
# File 'lib/resque/worker.rb', line 520 def started! redis.set("worker:#{self}:started", Time.now.to_s) end |
#startup ⇒ Object
Runs all the methods needed when a worker begins its lifecycle.
277 278 279 280 281 282 283 284 285 286 287 288 |
# File 'lib/resque/worker.rb', line 277 def startup Kernel.warn "WARNING: This way of doing signal handling is now deprecated. Please see http://hone.heroku.com/resque/2012/08/21/resque-signals.html for more info." unless term_child or $TESTING enable_gc_optimizations register_signal_handlers prune_dead_workers run_hook :before_first_fork register_worker # Fix buffering so we can `rake resque:work > resque.log` and # get output from the child in there. $stdout.sync = true end |
#state ⇒ Object
Returns a symbol representing the current worker state, which can be either :working or :idle
546 547 548 |
# File 'lib/resque/worker.rb', line 546 def state redis.exists("worker:#{self}") ? :working : :idle end |
#unpause_processing ⇒ Object
Start processing jobs again after a pause
409 410 411 412 |
# File 'lib/resque/worker.rb', line 409 def unpause_processing log "CONT received; resuming job processing" @paused = false end |
#unregister_signal_handlers ⇒ Object
326 327 328 329 330 331 332 333 334 335 336 |
# File 'lib/resque/worker.rb', line 326 def unregister_signal_handlers trap('TERM') { raise TermException.new("SIGTERM") } trap('INT', 'DEFAULT') begin trap('QUIT', 'DEFAULT') trap('USR1', 'DEFAULT') trap('USR2', 'DEFAULT') rescue ArgumentError end end |
#unregister_worker(exception = nil) ⇒ Object
Unregisters ourself as a worker. Useful when shutting down.
456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 |
# File 'lib/resque/worker.rb', line 456 def unregister_worker(exception = nil) # If we're still processing a job, make sure it gets logged as a # failure. if (hash = processing) && !hash.empty? job = Job.new(hash['queue'], hash['payload']) # Ensure the proper worker is attached to this job, even if # it's not the precise instance that died. job.worker = self job.fail(exception || DirtyExit.new) end redis.srem(:workers, self) redis.del("worker:#{self}") redis.del("worker:#{self}:started") Stat.clear("processed:#{self}") Stat.clear("failed:#{self}") end |
#validate_queues ⇒ Object
A worker must be given a queue, otherwise it won’t know what to do with itself.
You probably never need to call this.
107 108 109 110 111 |
# File 'lib/resque/worker.rb', line 107 def validate_queues if @queues.nil? || @queues.empty? raise NoQueueError.new("Please give each worker at least one queue.") end end |
#verbose ⇒ Object
638 639 640 641 |
# File 'lib/resque/worker.rb', line 638 def verbose logger_severity_deprecation_warning @verbose end |
#verbose=(value) ⇒ Object
648 649 650 651 652 653 654 655 656 657 658 |
# File 'lib/resque/worker.rb', line 648 def verbose=(value); logger_severity_deprecation_warning if value && !very_verbose Resque.logger.formatter = VerboseFormatter.new elsif !value Resque.logger.formatter = QuietFormatter.new end @verbose = value end |
#very_verbose ⇒ Object
643 644 645 646 |
# File 'lib/resque/worker.rb', line 643 def very_verbose logger_severity_deprecation_warning @very_verbose end |
#very_verbose=(value) ⇒ Object
660 661 662 663 664 665 666 667 668 669 670 671 |
# File 'lib/resque/worker.rb', line 660 def very_verbose=(value) logger_severity_deprecation_warning if value Resque.logger.formatter = VeryVerboseFormatter.new elsif !value && verbose Resque.logger.formatter = VerboseFormatter.new else Resque.logger.formatter = QuietFormatter.new end @very_verbose = value end |
#will_fork? ⇒ Boolean
540 541 542 |
# File 'lib/resque/worker.rb', line 540 def will_fork? !@cant_fork && !$TESTING && (ENV["FORK_PER_JOB"] != 'false') end |
#windows_worker_pids ⇒ Object
Returns an Array of string pids of all the other workers on this machine. Useful when pruning dead workers on startup.
588 589 590 591 |
# File 'lib/resque/worker.rb', line 588 def windows_worker_pids tasklist_output = `tasklist /FI "IMAGENAME eq ruby.exe" /FO list`.encode("UTF-8", Encoding.locale_charmap) tasklist_output.split($/).select { |line| line =~ /^PID:/}.collect{ |line| line.gsub /PID:\s+/, '' } end |
#work(interval = 5.0, &block) ⇒ Object
This is the main workhorse method. Called on a Worker instance, it begins the worker life cycle.
The following events occur during a worker’s life cycle:
-
Startup: Signals are registered, dead workers are pruned,
and this worker is registered. -
Work loop: Jobs are pulled from a queue and processed.
-
Teardown: This worker is unregistered.
Can be passed a float representing the polling frequency. The default is 5 seconds, but for a semi-active site you may want to use a smaller value.
Also accepts a block which will be passed the job as soon as it has completed processing. Useful for testing.
129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 |
# File 'lib/resque/worker.rb', line 129 def work(interval = 5.0, &block) interval = Float(interval) $0 = "resque: Starting" startup loop do break if shutdown? if not paused? and job = reserve log "got: #{job.inspect}" job.worker = self working_on job procline "Processing #{job.queue} since #{Time.now.to_i} [#{job.payload_class}]" if @child = fork(job) do unregister_signal_handlers if term_child reconnect perform(job, &block) exit! unless run_at_exit_hooks end srand # Reseeding procline "Forked #{@child} at #{Time.now.to_i}" begin Process.waitpid(@child) rescue SystemCallError nil end job.fail(DirtyExit.new($?.to_s)) if $?.signaled? else reconnect perform(job, &block) end done_working @child = nil else break if interval.zero? log! "Sleeping for #{interval} seconds" procline paused? ? "Paused" : "Waiting for #{@queues.join(',')}" sleep interval end end unregister_worker rescue Exception => exception log "Failed to start worker : #{exception.inspect}" unregister_worker(exception) end |
#worker_pids ⇒ Object
Returns an Array of string pids of all the other workers on this machine. Useful when pruning dead workers on startup.
578 579 580 581 582 583 584 |
# File 'lib/resque/worker.rb', line 578 def worker_pids if RUBY_PLATFORM =~ /solaris/ solaris_worker_pids else linux_worker_pids end end |
#working? ⇒ Boolean
Boolean - true if working, false if not
531 532 533 |
# File 'lib/resque/worker.rb', line 531 def working? state == :working end |
#working_on(job) ⇒ Object
Given a job, tells Redis we’re working on it. Useful for seeing what workers are doing and when.
477 478 479 480 481 482 483 |
# File 'lib/resque/worker.rb', line 477 def working_on(job) data = encode \ :queue => job.queue, :run_at => Time.now.utc.iso8601, :payload => job.payload redis.set("worker:#{self}", data) end |